from wagtail.core import blocks

from webspace.cms import constants
from webspace.cms.blocks.choice import ThemeChoiceBlock
from webspace.cms.blocks.common import \
    SvgWithSizeBlock, \
    TextBlock, \
    ButtonBlock, \
    EntryBlock, \
    ImageWithSizeBlock, \
    PersonBlock, \
    IconTextBlock


class StreamCard(blocks.StructBlock):
    streams = blocks.StreamBlock(
        [
            ('svg', SvgWithSizeBlock()),
            ('image', ImageWithSizeBlock()),
            ('text', TextBlock()),
            ('icon_text', IconTextBlock())
        ],
        required=False
    )

    class Meta:
        template = '%s/common/cards/stream_card.html' % constants.BLOCK_TEMPLATES_PATH
        label = "Stream Card"


# FIXME: remove CustomCard and merge it with StreamCard

class CustomCard(blocks.StructBlock):
    text = TextBlock()
    button = ButtonBlock()
    icon = SvgWithSizeBlock(label="Icon")
    media = blocks.StreamBlock(
        [
            ('svg', SvgWithSizeBlock()),
            ('image', ImageWithSizeBlock()),
        ],
        max_num=1,
        required=False
    )

    class Meta:
        template = '%s/common/cards/custom_card.html' % constants.BLOCK_TEMPLATES_PATH
        label = "Custom Card"


class PersonCard(PersonBlock):
    pass


class ReviewCard(blocks.StructBlock):
    score = blocks.IntegerBlock()
    text = TextBlock()
    media = blocks.StreamBlock(
        [
            ('svg', SvgWithSizeBlock()),
            ('image', ImageWithSizeBlock()),
        ],
        max_num=1,
        required=False
    )

    class Meta:
        template = '%s/common/cards/review_card.html' % constants.BLOCK_TEMPLATES_PATH
        label = "Review Card"


class CardsEntry(EntryBlock):
    amp_scripts = ['carousel']
    theme_reverse = ThemeChoiceBlock(required=False)
    theme_content = ThemeChoiceBlock(required=False)
    carousel = blocks.BooleanBlock(required=False)
    cards = blocks.StreamBlock(
        [
            ('custom', CustomCard()),
            ('person', PersonCard()),
            ('review', ReviewCard()),
            ('stream', StreamCard()),
        ],
        min_num=1
    )

    def mock(self, stop=None, carousel=True, *args, **kwargs):
        if 'theme' in kwargs:
            icon = self.SVG_ICON_LIGHT if kwargs['theme'] == constants.THEME_LIGHT \
                else self.SVG_ICON_SPACE
            file = self.SVG_SQUARE_LIGHT if kwargs['theme'] == constants.THEME_LIGHT \
                else self.SVG_SQUARE_SPACE

        else:
            icon = self.SVG_ICON_SPACE
            file = self.SVG_SQUARE_SPACE
        ret = {
            'type': 'cards',
            'value': {
                'cards': [],
                'carousel': carousel
            }
        }
        cci = {
            'text': {
                'value': self.xs,
                'align': 'center'
            },
            'button': self.button(constants.BUTTON_SECONDARY)
        }
        if self.random_counter % 2:
            cci['media'] = [{
                'type': 'svg',
                'value': {
                    'file': self.file(file).id,
                    'size': 'full'
                },
            }]
        else:
            cci['icon'] = {
                'file': self.file(icon).id,
                'size': 'l'
            }
        card_custom = [cci, cci, cci, cci, cci, cci, cci]
        i = 0
        for card_item in card_custom:
            if not carousel and stop and i == stop:
                break
            ret['value']['cards'].append({
                'type': 'custom',
                'value': {
                    'custom': card_item
                }
            })
            ret['value']['cards'][i]['value'] = card_item
            i += 1
        self.mock_data.update(ret)
        return super().mock(*args, **kwargs)

    class Meta:
        template = '%s/entries/cards.html' % constants.BLOCK_TEMPLATES_PATH
        label = "Cards"
        #  icon = 'image'



import re
from django.urls import include, path, reverse
from wagtail.admin.rich_text.converters import html_to_contentstate
from wagtail.admin.rich_text.converters.html_to_contentstate import (
    BlockElementHandler, KEEP_WHITESPACE, WHITESPACE_RE
)
import wagtail.admin.rich_text.editors.draftail.features as draftail_features
from wagtail.admin.rich_text.converters.html_to_contentstate import InlineStyleElementHandler
from wagtail.admin.menu import MenuItem
from wagtail.core import hooks

from webspace.bakery import urls as urls_bakery

NOTHING_RE = re.compile('a^')


#  RichText Mark

@hooks.register('register_rich_text_features')
def register_mark_feature(features):
    feature_name = 'mark'
    type_ = 'MARK'
    tag = 'mark'

    control = {
        'type': type_,
        'label': '☆',
        'description': 'Mark',
    }

    features.register_editor_plugin(
        'draftail', feature_name, draftail_features.InlineStyleFeature(control)
    )

    db_conversion = {
        'from_database_format': {tag: InlineStyleElementHandler(type_)},
        'to_database_format': {'style_map': {type_: tag}},
    }

    features.register_converter_rule('contentstate', feature_name, db_conversion)
    features.default_features.append('mark')


#  Shadow

@hooks.register('register_rich_text_features')
def register_shadow_feature(features):
    feature_name = 'shadow'
    type_ = 'SHADOW'
    tag = 'span'

    control = {
        'type': type_,
        'label': '+',
        'description': 'Shadow',
    }

    features.register_editor_plugin(
        'draftail', feature_name, draftail_features.InlineStyleFeature(control)
    )

    db_conversion = {
        'from_database_format': {tag: InlineStyleElementHandler(type_)},
        'to_database_format': {
            'style_map': {
                type_: {
                    'element': tag,
                    'props': {
                        'class': 'shadow',
                    },
                },
            },
        },
    }

    features.register_converter_rule('contentstate', feature_name, db_conversion)
    features.default_features.append('shadow')


#  RichText Code Blocks

class PreformattedTextElementHandler(BlockElementHandler):
    """
    BlockElementHandler that preserves all whitespace.
    """

    def handle_starttag(self, name, attrs, state, contentstate):
        super().handle_starttag(name, attrs, state, contentstate)
        # Keep all whitespace while rendering this block
        html_to_contentstate.WHITESPACE_RE = NOTHING_RE
        state.leading_whitespace = KEEP_WHITESPACE

    def handle_endtag(self, name, state, contentstate):
        # Reset whitespace handling to normal behaviour
        html_to_contentstate.WHITESPACE_RE = WHITESPACE_RE
        super().handle_endtag(name, state, contentstate)


@hooks.register('register_rich_text_features')
def register_code_block_feature(features):
    feature_name = 'code-block'
    feature_type = 'code-block'
    control = {
        'type': feature_type,
        'label': '{}',
        'description': 'Code',
    }
    features.register_editor_plugin(
        'draftail', feature_name, draftail_features.BlockFeature(control)
    )
    features.register_converter_rule('contentstate', feature_name, {
        'from_database_format': {
            'pre': PreformattedTextElementHandler(feature_type),
        },
        'to_database_format': {
            'block_map': {
                feature_type: {
                    'element': 'pre',
                    'props': {'class': 'code'},
                },
            },
        },
    })
    features.default_features.append(feature_name)


#  RichText Eternal Links

from django.utils.html import escape
from wagtail.core.rich_text import LinkHandler


class NoFollowExternalLinkHandler(LinkHandler):
    identifier = 'external'

    @classmethod
    def expand_db_attributes(cls, attrs):
        href = attrs["href"]

        # Transform `rel` and `target` attribute

        target = ''
        if '/target_blank' in href:
            target = 'target="_blank"'
            href = href.replace('/target_blank', '')

        rel = 'rel="nofollow"'
        if '/rel_follow' in href:
            rel = ''
            href = href.replace('/rel_follow', '')

        return '<a href="%s" %s %s>' % (escape(href), rel, target)


@hooks.register('register_rich_text_features')
def register_external_link(features):
    features.register_link_type(NoFollowExternalLinkHandler)



# Generated by Django 3.0.7 on 2020-07-06 14:50

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0004_person'),
    ]

    operations = [
        migrations.AddField(
            model_name='genericpage',
            name='amp_active',
            field=models.BooleanField(default=True),
        ),
        migrations.AlterField(
            model_name='blogpage',
            name='author',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='cms.Person'),
        ),
    ]



from datetime import date
from django.db import models
from django.conf import settings

from wagtail.admin.edit_handlers import FieldPanel
from wagtail.core import fields
from wagtail.core.models import Page
from wagtail.admin.edit_handlers import StreamFieldPanel
from wagtail.core.fields import StreamField
from wagtail.core import blocks
from wagtail.images.blocks import ImageChooserBlock
from wagtail.documents.blocks import DocumentChooserBlock

from modelcluster.fields import ParentalKey
from modelcluster.tags import ClusterTaggableManager
from taggit.models import TaggedItemBase

from webspace.bakery.abstract import WagtailPageBakeryModel
from webspace.cms import constants
from webspace.cms.amp.mixins import AmpMixin
from webspace.cms.forms.abstract import FormMixin
from webspace.loader import get_model

from ._mixins import TagMixin

GenericPage = get_model('cms', 'GenericPage')


class APortfolioPageTag(TaggedItemBase):
    content_object = ParentalKey('PortfolioPage', related_name='tagged_items')

    class Meta:
        app_label = 'cms'
        abstract = True


class APortfolioPage(AmpMixin, FormMixin, WagtailPageBakeryModel, GenericPage):
    template = '%s/portfolio_page.html' % constants.PAGES_TEMPLATES_PATH
    cover = StreamField([
        ('image', ImageChooserBlock()),
        ('svg', DocumentChooserBlock()),
    ], blank=True)
    tags = ClusterTaggableManager(through='PortfolioPageTag', blank=True)
    date = models.DateTimeField("Date publication + tri", default=date.today)
    intro_title = models.CharField(
        default='',
        max_length=500,
        blank=True,
        help_text="Titre d'introduction qui s'affiche sur un block portfolio de présentation"
    )
    intro = models.CharField(
        default='',
        max_length=500,
        blank=True,
        help_text="Texte d'introduction qui s'affiche sur un block portfolio de présentation"
    )

    related_portfolio = StreamField([
        ('portfolio', blocks.PageChooserBlock(required=False, target_model='cms.PortfolioPage')),
    ], blank=True)

    promote_panels = GenericPage.promote_panels + [
        StreamFieldPanel('schemas'),
    ]
    content_panels = [
                         StreamFieldPanel('cover'),
                         FieldPanel('date'),
                         FieldPanel('tags'),
                         FieldPanel('intro_title'),
                         FieldPanel('intro'),
                         StreamFieldPanel('related_portfolio'),
                     ] + GenericPage.content_panels

    class Meta:
        abstract = True
        app_label = 'cms'

    def get_absolute_url(self):
        return self.full_url


class APortfolioIndexPage(TagMixin, AmpMixin, WagtailPageBakeryModel, GenericPage):
    subpage_types = ['PortfolioPage']
    template = '%s/portfolio_index_page.html' % constants.PAGES_TEMPLATES_PATH
    bg_desktop = StreamField([
        ('image', ImageChooserBlock()),
        ('svg', DocumentChooserBlock()),
    ], blank=True)
    bg_mobile = StreamField([
        ('image', ImageChooserBlock()),
        ('svg', DocumentChooserBlock()),
    ], blank=True)
    first_text = fields.RichTextField(default='', blank=True, features=settings.RICH_TEXT_FEATURES)

    content_panels = [
                         StreamFieldPanel('bg_desktop'),
                         StreamFieldPanel('bg_mobile'),
                         FieldPanel('first_text'),
                     ] + Page.content_panels

    class Meta:
        abstract = True
        app_label = 'cms'



from wagtail.admin.edit_handlers import StreamFieldPanel

from webspace.cms.amp.mixins import AmpMixin
from webspace.bakery.abstract import WagtailPageBakeryModel
from webspace.cms import constants
from webspace.loader import get_model
from webspace.cms.forms.abstract import FormMixin

GenericPage = get_model('cms', 'GenericPage')


class AContentPage(AmpMixin, FormMixin, WagtailPageBakeryModel, GenericPage):
    template = '%s/content_page.html' % constants.PAGES_TEMPLATES_PATH

    promote_panels = GenericPage.promote_panels + [
        StreamFieldPanel('schemas'),
    ]

    class Meta:
        abstract = True
        app_label = 'cms'



import re
from django.utils.text import slugify
from django.conf import settings
from wagtail.documents.blocks import DocumentChooserBlock
from wagtail.images.blocks import ImageChooserBlock
from wagtail.core.rich_text import RichText
from wagtail.core import blocks
from wagtail.snippets.blocks import SnippetChooserBlock

from webspace.cms.blocks.mocker import Mocker
from webspace.cms import constants
from webspace.cms.amp.utils import amp_mode_active
from .choice import \
    ThemeChoiceBlock, \
    SizeChoiceBlock, \
    ContainerChoiceBlock, \
    ButtonChoiceBlock, \
    BackgroundPositionChoiceBlock

#  Add ids to headlines
__original__html__ = RichText.__html__
heading_re = r"<h(\d)[^>]*>([^<]*)</h\1>"


def add_id_attribute(match):
    n = match.group(1)
    text_content = match.group(2)
    id = slugify(text_content)
    return f'<h{n} id="titles-{id}">{text_content}</h{n}>'


def with_heading_ids(self):
    html = __original__html__(self)
    return re.sub(heading_re, add_id_attribute, html)


RichText.__html__ = with_heading_ids


class TextBlock(blocks.StructBlock):
    value = blocks.RichTextBlock(label="Text", required=False, features=settings.RICH_TEXT_FEATURES)

    class Meta:
        template = '%s/common/text.html' % constants.BLOCK_TEMPLATES_PATH
        label = "Text"


class SvgBlock(blocks.StructBlock):
    file = DocumentChooserBlock(required=False)


class SvgWithSizeBlock(SvgBlock):
    size = SizeChoiceBlock(required=False)

    class Meta:
        template = '%s/common/svg.html' % constants.BLOCK_TEMPLATES_PATH
        label = "Svg"


class ImageBlock(blocks.StructBlock):
    file = ImageChooserBlock(label="Image 500x500", required=False)


class ImageWithSizeBlock(ImageBlock):
    size = SizeChoiceBlock(required=False)

    class Meta:
        template = '%s/common/picture_sizer.html' % constants.BLOCK_TEMPLATES_PATH
        label = "Image"


class EmbedBlock(blocks.StructBlock):
    link = blocks.URLBlock(required=False)


class EmbedWithSizeBlock(EmbedBlock):
    size = SizeChoiceBlock(required=False)

    class Meta:
        template = '%s/common/embed.html' % constants.BLOCK_TEMPLATES_PATH
        label = "Embed"


class BackgroundBlock(blocks.StructBlock):
    desktop = SvgBlock()
    mobile = SvgBlock()


class LinkBlock(blocks.StructBlock):
    text = blocks.CharBlock(required=False)
    link = blocks.URLBlock(required=False, label="Button link")
    page = blocks.PageChooserBlock(required=False, label="Button Page")
    open_new_tab = blocks.BooleanBlock(default=False, required=False, label="Button nouvel onglet")

    class Meta:
        template = '%s/common/link.html' % constants.BLOCK_TEMPLATES_PATH
        label = "Link"


class ButtonBlock(LinkBlock):
    type = ButtonChoiceBlock(required=False, label="Button type")

    class Meta:
        template = '%s/common/button.html' % constants.BLOCK_TEMPLATES_PATH
        label = "Button"


class ButtonsBlock(blocks.StreamBlock):
    button = ButtonBlock()

    class Meta:
        template = '%s/common/buttons.html' % constants.BLOCK_TEMPLATES_PATH
        required = True


class IconTextBlock(blocks.StructBlock):
    icon = SnippetChooserBlock('cms.IconSnippet', required=True)
    text = TextBlock()

    class Meta:
        template = '%s/common/icon_text.html' % constants.BLOCK_TEMPLATES_PATH
        label = "Icon Text"


class FormBlock(blocks.StructBlock):
    form = SnippetChooserBlock('cms.Form', required=True)

    class Meta:
        template = '%s/common/form.html' % constants.BLOCK_TEMPLATES_PATH
        label = "Form"


class GalleryBlock(blocks.StructBlock):
    gallery = SnippetChooserBlock('cms.Gallery', required=True)

    class Meta:
        template = '%s/common/gallery.html' % constants.BLOCK_TEMPLATES_PATH
        label = "Gallery"


class GallerySliderBlock(GalleryBlock):

    class Meta:
        template = '%s/common/gallery_slider.html' % constants.BLOCK_TEMPLATES_PATH
        label = "Gallery Slider"


class PersonBlock(blocks.StructBlock):
    person = SnippetChooserBlock('cms.Person', required=True)

    class Meta:
        template = '%s/common/cards/person.html' % constants.BLOCK_TEMPLATES_PATH
        label = "Person"


class EntryBlock(Mocker, blocks.StructBlock):
    bg_desktop = blocks.StreamBlock(
        [
            ('image', ImageChooserBlock()),
            ('svg', DocumentChooserBlock()),
        ],
        max_num=5,
        required=False
    )
    bg_mobile = blocks.StreamBlock(
        [
            ('image', ImageChooserBlock()),
            ('svg', DocumentChooserBlock()),
        ],
        max_num=5,
        required=False
    )
    svg_bg_position = BackgroundPositionChoiceBlock(
        required=False,
        default=constants.BACKROUND_POSITION_CENTER,
        label="Bg Position"
    )
    bg_layer = blocks.BooleanBlock(default=False, required=False)
    container = ContainerChoiceBlock(required=False)
    padding_top = blocks.BooleanBlock(default=True, required=False)
    padding_bottom = blocks.BooleanBlock(default=True, required=False)
    theme = ThemeChoiceBlock(required=False)

    def get_template(self, context=None):
        if amp_mode_active():
            return self.meta.template.replace('.html', '_amp.html')
        return self.meta.template
